@@ -36,8 +36,9 @@ fn counts(c: &mut Criterion) {
     drop(inputs);
 
     let mut group = c.benchmark_group("counts");
-    group.sample_size(30);
-    group.measurement_time(Duration::from_secs(10));
+    group.sample_size(10);
+    group.measurement_time(Duration::from_secs(5));
+    group.warm_up_time(Duration::from_secs(1));
     for set in input_sets.iter() {
         let l = set.len();
         group.throughput(Throughput::Elements(l as u64));
@@ -96,8 +97,9 @@ fn scanning_sort(c: &mut Criterion) {
     drop(inputs);
 
     let mut group = c.benchmark_group("scanning_sort_level_4");
-    group.sample_size(30);
-    group.measurement_time(Duration::from_secs(10));
+    group.sample_size(10);
+    group.measurement_time(Duration::from_secs(5));
+    group.warm_up_time(Duration::from_secs(1));
     for set in input_sets.iter() {
         let l = set.len();
         group.throughput(Throughput::Elements(l as u64));
@@ -108,7 +110,7 @@ fn scanning_sort(c: &mut Criterion) {
                 bench.iter_batched(
                     || set.clone(),
                     |mut input| {
-                        scanning_radix_sort(&tuning, &mut input, 0);
+                        scanning_radix_sort(&tuning, &mut input, 0, true);
                         black_box(input);
                     },
                     BatchSize::SmallInput,
@@ -120,7 +122,7 @@ fn scanning_sort(c: &mut Criterion) {
             bench.iter_batched(
                 || set.clone(),
                 |mut input| {
-                    lsb_radix_sort_adapter(&tuning, &mut input, 3, 0);
+                    lsb_radix_sort_adapter(&mut input, 3, 0, true);
                     black_box(input);
                 },
                 BatchSize::SmallInput,
@@ -157,8 +159,9 @@ fn ska_sort(c: &mut Criterion) {
     drop(inputs);
 
     let mut group = c.benchmark_group("ska_sort_level_4");
-    group.sample_size(30);
-    group.measurement_time(Duration::from_secs(10));
+    group.sample_size(10);
+    group.measurement_time(Duration::from_secs(5));
+    group.warm_up_time(Duration::from_secs(1));
     for set in input_sets.iter() {
         let l = set.len();
         group.throughput(Throughput::Elements(l as u64));
@@ -177,7 +180,7 @@ fn ska_sort(c: &mut Criterion) {
             bench.iter_batched(
                 || set.clone(),
                 |mut input| {
-                    lsb_radix_sort_adapter(&tuning, &mut input, 3, 0);
+                    lsb_radix_sort_adapter(&mut input, 3, 0, true);
                     black_box(input);
                 },
                 BatchSize::SmallInput,
@@ -209,8 +212,9 @@ fn ska_sort(c: &mut Criterion) {
     drop(inputs);
 
     let mut group = c.benchmark_group("ska_sort_level_8");
-    group.sample_size(30);
-    group.measurement_time(Duration::from_secs(10));
+    group.sample_size(10);
+    group.measurement_time(Duration::from_secs(5));
+    group.warm_up_time(Duration::from_secs(1));
     for set in input_sets.iter() {
         let l = set.len();
         group.throughput(Throughput::Elements(l as u64));
@@ -229,7 +233,7 @@ fn ska_sort(c: &mut Criterion) {
             bench.iter_batched(
                 || set.clone(),
                 |mut input| {
-                    lsb_radix_sort_adapter(&tuning, &mut input, 3, 0);
+                    lsb_radix_sort_adapter(&mut input, 3, 0, true);
                     black_box(input);
                 },
                 BatchSize::SmallInput,
