@@ -6,6 +6,7 @@ use crate::RadixKey;
 use arbitrary_chunks::ArbitraryChunks;
 use rayon::prelude::*;
 use std::cmp::min;
+use std::ptr::copy_nonoverlapping;
 use std::sync::Mutex;
 
 struct ScannerBucketInner<'a, T> {
@@ -22,11 +23,11 @@ struct ScannerBucket<'a, T> {
 
 #[inline]
 fn get_scanner_buckets<'a, T>(
-    counts: &Vec<usize>,
+    counts: &[usize; 256],
     bucket: &'a mut [T],
 ) -> Vec<ScannerBucket<'a, T>> {
     let mut out: Vec<_> = bucket
-        .arbitrary_chunks_mut(counts.clone())
+        .arbitrary_chunks_mut(counts.to_vec())
         .enumerate()
         .map(|(index, chunk)| ScannerBucket {
             index,
@@ -55,12 +56,14 @@ fn scanner_thread<T>(
     let mut stash: Vec<Vec<T>> = Vec::with_capacity(256);
     stash.resize(256, Vec::with_capacity(128));
     let mut finished_count = 0;
-    let mut finished_map: Vec<bool> = vec![false; 256];
+    let mut finished_map = [false; 256];
 
     'outer: loop {
         for m in scanner_buckets {
-            if finished_map[m.index] {
-                continue;
+            unsafe {
+                if *finished_map.get_unchecked(m.index) {
+                    continue;
+                }
             }
 
             let mut guard = match m.inner.try_lock() {
@@ -89,24 +92,24 @@ fn scanner_thread<T>(
                 let chunks = read_data.chunks_exact(8);
                 let rem = chunks.remainder();
 
-                chunks.into_iter().for_each(|chunk| {
-                    let a = chunk[0].get_level(level) as usize;
-                    let b = chunk[1].get_level(level) as usize;
-                    let c = chunk[2].get_level(level) as usize;
-                    let d = chunk[3].get_level(level) as usize;
-                    let e = chunk[4].get_level(level) as usize;
-                    let f = chunk[5].get_level(level) as usize;
-                    let g = chunk[6].get_level(level) as usize;
-                    let h = chunk[7].get_level(level) as usize;
+                chunks.into_iter().for_each(|chunk| unsafe {
+                    let a = chunk.get_unchecked(0).get_level(level) as usize;
+                    let b = chunk.get_unchecked(1).get_level(level) as usize;
+                    let c = chunk.get_unchecked(2).get_level(level) as usize;
+                    let d = chunk.get_unchecked(3).get_level(level) as usize;
+                    let e = chunk.get_unchecked(4).get_level(level) as usize;
+                    let f = chunk.get_unchecked(5).get_level(level) as usize;
+                    let g = chunk.get_unchecked(6).get_level(level) as usize;
+                    let h = chunk.get_unchecked(7).get_level(level) as usize;
 
-                    stash[a].push(chunk[0]);
-                    stash[b].push(chunk[1]);
-                    stash[c].push(chunk[2]);
-                    stash[d].push(chunk[3]);
-                    stash[e].push(chunk[4]);
-                    stash[f].push(chunk[5]);
-                    stash[g].push(chunk[6]);
-                    stash[h].push(chunk[7]);
+                    stash.get_unchecked_mut(a).push(*chunk.get_unchecked(0));
+                    stash.get_unchecked_mut(b).push(*chunk.get_unchecked(1));
+                    stash.get_unchecked_mut(c).push(*chunk.get_unchecked(2));
+                    stash.get_unchecked_mut(d).push(*chunk.get_unchecked(3));
+                    stash.get_unchecked_mut(e).push(*chunk.get_unchecked(4));
+                    stash.get_unchecked_mut(f).push(*chunk.get_unchecked(5));
+                    stash.get_unchecked_mut(g).push(*chunk.get_unchecked(6));
+                    stash.get_unchecked_mut(h).push(*chunk.get_unchecked(7));
                 });
 
                 rem.into_iter().for_each(|v| {
@@ -132,7 +135,14 @@ fn scanner_thread<T>(
             let end = guard.write_head + to_write;
             let start = guard.write_head;
 
-            guard.chunk[start..end].copy_from_slice(&some);
+            unsafe {
+                copy_nonoverlapping(
+                    some.get_unchecked(0),
+                    guard.chunk.get_unchecked_mut(start),
+                    end - start,
+                );
+            }
+
             guard.write_head += to_write;
 
             if guard.write_head >= m.len as usize {
@@ -150,15 +160,16 @@ fn scanner_thread<T>(
 // scanning_radix_sort does a parallel MSB-first sort. Following this, depending on the number of
 // elements remaining in each bucket, it will either do an MSB-sort or an LSB-sort, making this
 // a dynamic hybrid sort.
-pub fn scanning_radix_sort<T>(tuning: &TuningParameters, bucket: &mut [T], level: usize)
+pub fn scanning_radix_sort<T>(tuning: &TuningParameters, bucket: &mut [T], start_level: usize, parallel_count: bool)
 where
-    T: RadixKey + Sized + Send + Ord + Copy + Sync,
+    T: RadixKey + Sized + Send + Copy + Sync,
 {
-    let msb_counts = if level == 0 && bucket.len() > tuning.par_count_threshold {
-        par_get_counts(bucket, level)
+    let (msb_counts, level) = if let Some(s) = get_counts_and_level(bucket, start_level, T::LEVELS - 1, parallel_count) {
+        s
     } else {
-        get_counts(bucket, level)
+        return;
     };
+
     let scanner_buckets = get_scanner_buckets(&msb_counts, bucket);
     let cpus = num_cpus::get();
     let threads = min(cpus, scanner_buckets.len());
@@ -177,13 +188,13 @@ where
     }
 
     bucket
-        .arbitrary_chunks_mut(msb_counts)
+        .arbitrary_chunks_mut(msb_counts.to_vec())
         .par_bridge()
         .for_each(|c| {
             if c.len() > tuning.ska_sort_threshold {
                 msb_ska_sort(tuning, c, level + 1);
             } else {
-                lsb_radix_sort_adapter(tuning, c, T::LEVELS - 1, level + 1);
+                lsb_radix_sort_adapter(c, T::LEVELS - 1, level + 1, false);
             }
         });
 }
